


import datetime


def edm_action_date(datetime: datetime.datetime) -> str:
    return datetime.strftime("%Y-%m-%dT%H:%M:%S")


def get_new_invoice_number(invoice_number_prefix: str, last_invoice_number: str) -> str:
    
    MAX_INVOICE_NUMBER_COUNT = 7
    invoice_number = str(int(last_invoice_number[7:]) + 1)
    invoice_number_str = "0" * (MAX_INVOICE_NUMBER_COUNT - len(invoice_number)) + invoice_number
        
    
    return f"{invoice_number_prefix}{datetime.datetime.now().strftime('%Y')}{invoice_number_str}"



from .models import Accounting, AccountingType, Address, Contact, Input, InvoiceLine, Invoice, Request, Response, Note, TaxTotal, TaxSubTotal, TaxCategory, TaxScheme, LegalMonetaryTotal, InvoiceType, LineUnitCode, Return


from time import sleep
from xml.etree import ElementTree
import requests
from zeep.client import Client
import datetime
from IsNetHelper.utils.utils import get_new_invoice_number
from IsNetHelper.models.Request import ArchiveInvoiceDetails, EArchiveInvoices, EArchiveInvoice, EArchiveInvoiceDetail, PagingRequest, Product, Receiver, ReceiverAddress, SearchArchiveInvoiceResultSet, SendArchiveInvoiceRequest, SendArchiveInvoiceRequestWithoutInvoiceNumber

class IsNetHelper:
    
    def __init__(self) -> None:
        self.session = requests.Session()
        self.INVOICE_WSDL_URL = self.get_invoice_wsdl_url(is_test=True)
        self.client = Client(self.INVOICE_WSDL_URL)
        self.address_client = Client(self.get_address_wsdl_url(is_test=True))
        
    def get_invoice_wsdl_url(self, is_test):
        if is_test:
            return "http://einvoiceservicetest.isnet.net.tr/InvoiceService/ServiceContract/InvoiceService.svc?singleWsdl"
        return "https://einvoiceservice.isnet.net.tr/InvoiceService/ServiceContract/InvoiceService.svc?singleWsdl"


    def get_address_wsdl_url(self, is_test):
        if is_test:
            return "http://einvoiceservicetest.isnet.net.tr/AddressBookService/ServiceContract/AddressBookService.svc?singleWsdl"
        return "http://einvoiceservice.isnet.net.tr/AddressBookService/ServiceContract/AddressBookService.svc?singleWsdl"


    def request(self, type: str, url: str, data: str, **kwargs):
        headers = kwargs.pop("headers", {})
        
        headers["Content-Type"] = "text/xml; charset=utf-8"
        return self.session.request(type, url, data=data, headers=headers, **kwargs)
    
    def search_archive_invoice(self, company_tax_code: str, is_cancelled: bool = False, paging_request: PagingRequest = PagingRequest(PageNumber=1, RecordsPerPage=1), result_set: SearchArchiveInvoiceResultSet = SearchArchiveInvoiceResultSet(
        IsAdditionalTaxIncluded=False,
        IsArchiveIncluded=False,
        IsAttachmentIncluded=False,
        IsExternalUrlIncluded=False,
        IsHtmlIncluded=False,
        IsInvoiceDetailIncluded=False,
        IsPDFIncluded=True,
        IsTextIncluded=False
    ), ExternalArchiveInvoiceCode: str = None):
        request = self.client.get_type("ns2:SearchArchiveInvoiceRequest")()  # WSDL'deki tam tipi alın
        request.CompanyTaxCode = company_tax_code
        request.IsCancelled = is_cancelled
        request.PagingRequest = paging_request.dict()
        request.ResultSet = result_set.dict()
        request.ExternalArchiveInvoiceCode = ExternalArchiveInvoiceCode
        return self.client.service.SearchArchiveInvoice(request=request)

    
    def get_cities(self):
        return self.address_client.service.GetCityList()
    
    def send_archive_invoice(self, request: SendArchiveInvoiceRequest):
        response = self.client.service.SendArchiveInvoice(request=request.dict())
        return response


    def get_last_invoice_number(self, company_tax_code: str):
        return self.search_archive_invoice(company_tax_code=company_tax_code, paging_request=PagingRequest(PageNumber=1, RecordsPerPage=1), result_set=SearchArchiveInvoiceResultSet()).ArchiveInvoices.ArchiveInvoice[0].ExternalArchiveInvoiceCode

    def send_archive_invoice_xml_without_invoice_number(self, request: SendArchiveInvoiceRequest):
        xml = ElementTree.tostring(request.to_xml(invoice_to_base64=True), encoding="utf-8")
        return self.client.service.SendArchiveInvoiceXmlWithoutInvoiceNumber(request=xml)


            


from .Accounting import Accounting, AccountingType
from .Address import Address
from .Contact import Contact
from .Input import Input
from .Invoice import Invoice, InvoiceType
from .InvoiceLine import InvoiceLine, LineUnitCode
from .Note import Note
from .Request import Request
from .Response import Response, Return
from .LegalMonetaryTotal import LegalMonetaryTotal
from .Tax import TaxCategory, TaxScheme, TaxSubTotal, TaxTotal


import setuptools
from setuptools import find_packages

setuptools.setup(
    name="isnet-helper",
    version="1.0.0",
    author="Esat Yılmaz",
    author_email="esatyilmaz3500@gmail.com",
    description="Isnet Helper",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.9',
    install_requires=[
        "pydantic", "requests", "zeep"
    ]
)



import base64
from typing import List, Optional
from xml.etree import ElementTree as ET
import pydantic
from pydantic import BaseModel

class CustomBaseModel(pydantic.BaseModel):
    
    def to_xml(self) -> str:
        raise NotImplementedError


class RequestHeader(CustomBaseModel):
    value: str


class PagingRequest(CustomBaseModel):
    PageNumber: int = 1
    RecordsPerPage: int = 25
    
    def to_xml(self) -> str:
        
        root = ET.Element("ein:PagingRequest")
        for field, value in self.dict().items():
            ET.SubElement(root, f"ein:{field}").text = str(value)
        return root
        
    
    


class SearchArchiveInvoiceResultSet(CustomBaseModel):
    IsAdditionalTaxIncluded: bool = False
    IsArchiveIncluded: bool = False
    IsAttachmentIncluded: bool = False
    IsExternalUrlIncluded: bool = False
    IsHtmlIncluded: bool = False
    IsInvoiceDetailIncluded: bool = False
    IsPDFIncluded: bool = False
    IsXMLIncluded: bool = False
    
    def to_xml(self) -> str:
        root = ET.Element("ein:SearchArchiveInvoiceResultSet")
        for field, value in self.dict().items():
            ET.SubElement(root, f"ein:{field}").text = str(value)
        return root

class SearchArchiveInvoiceRequestData(CustomBaseModel):
    CompanyTaxCode: str
    IsCancelled: bool = False
    PagingRequest: PagingRequest
    ResultSet: SearchArchiveInvoiceResultSet
    
    
    def to_xml(self) -> ET.Element:
        root = ET.Element("tem:request")
        ET.SubElement(root, "ein:CompanyTaxCode").text = self.CompanyTaxCode
        ET.SubElement(root, "ein:IsCancelled").text = str(self.IsCancelled).lower()
        root.append(self.PagingRequest.to_xml())
        root.append(self.ResultSet.to_xml())
        return root
    

class SearchArchiveInvoiceRequest(CustomBaseModel):
    headers: List[RequestHeader] = []
    request: SearchArchiveInvoiceRequestData
    
    def to_xml(self) -> str:
        root = ET.Element("soapenv:Envelope", attrib={"xmlns:soapenv": "http://schemas.xmlsoap.org/soap/envelope/", "xmlns:tem": "http://tempuri.org/", "xmlns:ein": "http://schemas.datacontract.org/2004/07/EInvoice.Service.Model", "xmlns:ein1": "http://schemas.datacontract.org/2004/07/EInvoice.Service.Model.DataTypes.Enums"})
        
        headers = ET.SubElement(root, "soapenv:Header")
        for header in self.headers:
            headers.append(header.to_xml())
        
        body = ET.SubElement(root, "soapenv:Body")
        
        ET.SubElement(body, "tem:SearchArchiveInvoice").append(self.request.to_xml())
        
        
        return root
    

class Product(BaseModel):
    __xml_name__ = "Product"
    ExternalProductCode: str
    MeasureUnit: str
    ProductCode: str
    ProductName: str
    ReceiverProductCode: str
    UnitPrice: float

class EArchiveInvoiceDetail(BaseModel):
    __xml_name__ = "InvoiceDetail"
    CurrencyCode: str
    DiscountAmount: float
    DiscountRate: float
    LineExtensionAmount: float
    Note: str
    Product: Product
    Quantity: float
    SpecialBasisAmount: float
    SpecialBasisPercent: float
    SpecialBasisTaxAmount: float
    StockDescription: str
    VATAmount: float
    VATRate: float
    TaxExemptionReasonCode: str

class ArchiveInvoiceDetails(BaseModel):
    __xml_name__ = "ArchiveInvoiceDetail"
    ArchiveInvoiceDetail: List['EArchiveInvoiceDetail']

class EArchiveInvoices(BaseModel):
    __xml_name__ = "ArchiveInvoices"
    ArchiveInvoice: List['EArchiveInvoice']

class ReceiverAddress(BaseModel):
    __xml_name__ = "Address"
    CityCode: str
    EMail: str

class Receiver(BaseModel):
    __xml_name__ = "Receiver"
    Address: ReceiverAddress
    ReceiverName: str
    ReceiverTaxCode: str
    RecipientType: str
    SendingType: str

class EArchiveInvoice(BaseModel):
    __xml_name__ = "ArchiveInvoice"
    CurrencyCode: str
    ExternalArchiveInvoiceCode: Optional[str]
    InvoiceCreationDate: str
    InvoiceDate: str
    InvoiceDetails: ArchiveInvoiceDetails
    InvoiceType: str
    Notes: List[str]
    Receiver: Receiver
    TaxExemptionReason: str
    TotalDiscountAmount: float
    TotalLineExtensionAmount: float
    TotalPayableAmount: float
    TotalTaxInclusiveAmount: float
    TotalVATAmount: float
    CrossRate: float

class SendArchiveInvoiceRequest(BaseModel):
    __xml_name__ = "request"
    ArchiveInvoices: EArchiveInvoices
    CompanyTaxCode: str
    
    def to_xml(self, invoice_to_base64: bool = False):
        envelope = ET.Element("soapenv:Envelope", {
        "xmlns:soapenv": "http://schemas.xmlsoap.org/soap/envelope/",
        "xmlns:tem": "http://tempuri.org/",
        "xmlns:ein": "http://schemas.datacontract.org/2004/07/EInvoice.Service.Model",
        "xmlns:arr": "http://schemas.microsoft.com/2003/10/Serialization/Arrays",
    })
        body = ET.SubElement(envelope, "soapenv:Body")
        send_archive_invoice = ET.SubElement(body, "tem:SendArchiveInvoice")
        request = ET.SubElement(send_archive_invoice, "tem:request")

        # ArchiveInvoices
        archive_invoices = ET.SubElement(request, "ein:ArchiveInvoices")
        for invoice in self.ArchiveInvoices.ArchiveInvoice:
            
            invoice_element = ET.Element("ein:ArchiveInvoice")
            for key, value in invoice.dict().items():
                if isinstance(value, list):
                    if key == "Notes":
                        notes = ET.SubElement(invoice_element, "ein:Notes")
                        for note in value:
                            ET.SubElement(notes, "ein:string").text = note
                elif isinstance(value, dict):
                    sub_element = ET.SubElement(invoice_element, f"ein:{key}")
                    for sub_key, sub_value in value.items():
                        ET.SubElement(sub_element, f"ein:{sub_key}").text = str(sub_value)
                else:
                    ET.SubElement(invoice_element, f"ein:{key}").text = str(value)
            
            
            archive_invoices.append(invoice_element)
        # CompanyTaxCode
        ET.SubElement(request, "ein:CompanyTaxCode").text = self.CompanyTaxCode

        return envelope

class EArchiveInvoiceWithoutInvoiceNumber(BaseModel):
    __xml_name__ = "ArchiveInvoice"
    ArchiveInvoiceContent: bytes
    SendMailAutomatically: bool
    
class SendArchiveInvoiceRequestWithoutInvoiceNumber(BaseModel):
    __xml_name__ = "request"
    ArchiveInvoices: EArchiveInvoices
    CompanyVendorNumber: str
    SendMailAutomatically: bool
    
    def to_xml(self):
        envelope = ET.Element("soapenv:Envelope", {
        "xmlns:soapenv": "http://schemas.xmlsoap.org/soap/envelope/",
        "xmlns:tem": "http://tempuri.org/",
        "xmlns:ein": "http://schemas.datacontract.org/2004/07/EInvoice.Service.Model",
        "xmlns:arr": "http://schemas.microsoft.com/2003/10/Serialization/Arrays",
    })
        body = ET.SubElement(envelope, "soapenv:Body")
        send_archive_invoice = ET.SubElement(body, "tem:SendArchiveInvoice")
        request = ET.SubElement(send_archive_invoice, "tem:request")

        # ArchiveInvoices
        archive_invoices = ET.Element("ein:ArchiveInvoices")
        for invoice in self.ArchiveInvoices.ArchiveInvoice:
            invoice_element = ET.Element("ein:ArchiveInvoice")
            for key, value in invoice.dict().items():
                if isinstance(value, list):
                    if key == "Notes":
                        notes = ET.SubElement(invoice_element, "ein:Notes")
                        for note in value:
                            ET.SubElement(notes, "ein:string").text = note
                elif isinstance(value, dict):
                    sub_element = ET.SubElement(invoice_element, f"ein:{key}")
                    for sub_key, sub_value in value.items():
                        ET.SubElement(sub_element, f"ein:{sub_key}").text = str(sub_value)
                else:
                    ET.SubElement(invoice_element, f"ein:{key}").text = str(value)
            
            earchive_invoice = ET.SubElement(archive_invoices, "ein:ArchiveInvoice")
            ET.SubElement(earchive_invoice, "ein:ArchiveInvoiceContent").text = base64.b64encode(ET.tostring(invoice_element))
            ET.SubElement(earchive_invoice, "ein:SendMailAutomatically").text = str(self.SendMailAutomatically).lower()
            
        # CompanyTaxCode
        ET.SubElement(body, "ein:CompanyVendorNumber").text = self.CompanyVendorNumber
        return ET.tostring(envelope, encoding="utf-8", method="xml")



EArchiveInvoices.update_forward_refs()